@@ -360,6 +360,7 @@ def main():
         tello = DroneBlocksTello()
         tello.connect()
         speed = tello.get_speed()
+        time.sleep(0.5)
 
         # If the user wants to see the Tello video from the handler
         # or the raw video from the Tello AND we are not simulating
@@ -389,8 +390,10 @@ def main():
         # ---------------------------- Initialize Web Server --------
         # -----------------------------------------------------------
         if start_tello_web:
+            print(f"Starting Tello Web from Tello Script Runner...")
+
             p2 = threading.Thread(target=web_main,
-                                  args=(tello, stop_event, web_port
+                                  args=(tello, stop_event, web_port, None
                                         ))
             p2.setDaemon(True)
             p2.start()
